# -*- coding: utf-8 -*-
import json

from fabric.api import env

from .exceptions import EnvironmentNotDefinedError
from .tasks import ulocal


def set_env(env_name, json_path):
    """
    Creates a dynamic environment based on the contents of the given
    json_path. Note that a this task relies on a "vagrant" environment defined
    on your environments json file to populate the task's env with the proper
    Vagrant's identity file.

    This task must be used to create your own envionment task in your fabfile,
    for example:

        # /path/to/my/environments/file.json
        {
            "vagrant": {
                "some_property": "some_value",
                "another_property": "another_value"
            },
            "staging": {
                ...
            }
        }


        # fabfile.py
        from fabric.api import task
        from fabutils.env import set_env


        @task
        def environment(env_name):
            set_env(env_name, '/path/to/my/environments/file.json')

        @task
        def some_task():
            ...


    And call it as follows:

        fab envinronment:vagrant some_task
    """
    with open(json_path, 'r') as data:
        try:
            environment = json.load(data)[env_name]

        except KeyError:
            raise EnvironmentNotDefinedError(
                "The environment '{0}' is not defined in file '{1}'".format(
                    env_name, json_path
                )
            )

    # Update the env with the Vagrant's identity file if the given env_name is
    # "vagrant" and no key_filename property is defined in the current env.
    if env_name == 'vagrant' and 'key_filename' not in environment:
        result = ulocal('vagrant ssh-config | grep IdentityFile', capture=True)
        env.key_filename = result.split()[1].replace('"', '')

    # Prepend the command "source" to each one of the commands defined in the
    # command_prefixes property of the json file.
    prefixes = environment.get('command_prefixes', [])
    sourced_prefixes = map(lambda p: 'source %s' % p, prefixes)
    environment.update(command_prefixes=sourced_prefixes)

    env.update(environment)



# -*- coding: utf-8 -*-
from fabric.api import local, run
from fabric.contrib.project import rsync_project

from .decorators import without_prefixes


# Decorated "local command" to run without command prefixes
ulocal = without_prefixes(local)


# Decorated "rsync_project" command to run without command prefixes
ursync_project = without_prefixes(rsync_project)


# Decorated "run" command to run without command prefixes
urun = without_prefixes(run)



# -*- coding: utf-8 -*-


class Error(Exception):
    """
    Base error exception for this module.
    """
    pass


class EnvironmentNotDefinedError(Error):
    """
    Exception that must be raised when a environment is not defined in the
    environments json file.
    """
    pass



# -*- coding: utf-8 -*-
from fabric.api import env


def without_prefixes(task):
    """
    Decorator that strips the ```command_prefixes``` attribute from the given
    task env to run it without calling those prefixes.

    This decorator is intended to create tasks that does not need to call the
    command_prefixes defined in the current env, for example if you have
    prefixes that need to be called inside a VM but not in your local machine.
    """
    def wrapper(*args, **kwargs):
        # Save the current env's command_prefixes and remove it from env
        command_prefixes = env.get('command_prefixes', [])
        env.command_prefixes = []

        # Call the given task
        result = task(*args, **kwargs)

        # Restore the original command_prefixes
        env.command_prefixes = command_prefixes

        return result

    return wrapper



# -*- coding: utf-8 -*-
from distutils.util import strtobool


def boolean(value):
    """
    Translate the given value to a boolean object.

    Example:
        >>> boolean(True)
        True

        >>> boolean(0)
        False

        >>> boolean('false')
        False

        >>> boolean('yes')
        True
    """
    if isinstance(value, bool):
        return value

    elif isinstance(value, int):
        return bool(value)

    else:
        return bool(strtobool(str(value)))


def join(*args):
    """
    Return a string composed of the result of applying str() to each one of
    the given args, separated by spaced, but only if the item exists.

    Example:
        >>> join('Hello', False, 1, True, 0, 'world')
        'Hello 1 True world'
    """
    strings = [str(arg) for arg in args if arg]

    return ' '.join(strings)


def arguments(*args, **kwargs):
    """
    Translates the given args and kwargs into a string composed of each one of
    the provided args followed by the key and value of each one of the
    provided kwargs' items in the form key=value, separated by spaces.

    Example:
        >>> arguments('arg1', 'arg2', kwarg1='val1', kwarg2='val2')
        'arg1 arg2 kwarg1=val1 kwarg2=val2'
    """
    named_arguments = ['{0}={1}'.format(*pair) for pair in kwargs.items()]
    arguments = [arg for arg in args] + named_arguments

    return join(*arguments)


def options(**kwargs):
    """
    Translates the given kwargs insto a string composed by the key of each one
    of the provided kwargs items in the form --key only if the corresponding
    value can be avalueated to a boolean True, separated by spaces.

    Example:
        >>> options(option1=True, option2='No', option3=1, option4=False)
        '--option1 --option3'
    """
    options = ['--{0}'.format(k) for k, v in kwargs.items() if boolean(v)]

    return join(*options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

from fabutils import __author__, __version__


setup(
    name='vo-fabutils',
    author=__author__,
    version=__version__,
    description=('Utilities for creating fabric tasks that deals with '
                 'Vagrant, Django, and more.'),
    long_description=open(os.path.realpath(os.path.join(
        os.path.dirname(__file__), 'README.rst'))).read(),
    author_email='desarrollo@vincoorbis.com',
    install_requires=[
        'fabric >= 1.10'
    ],
    packages=find_packages(),
    include_package_data=True,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
    ]
)
